@@ -132,14 +132,49 @@ DF = data.frame(val = 1:10, bool = TRUE, big = LETTERS[1:10],
                 stringsAsFactors = FALSE)
 
 rhandsontable(DF) %>%
-  hot_table(allowRowEdit = FALSE, allowColEdit = FALSE)
+  hot_context_menu(allowRowEdit = FALSE, allowColEdit = FALSE)
 
 ## ----fig.height = 3------------------------------------------------------
 MAT = matrix(rnorm(50), nrow = 10, dimnames = list(LETTERS[1:10],
                                                    letters[1:5]))
 
 rhandsontable(MAT) %>%
-  hot_table(exportToCsv = TRUE)
+  hot_context_menu(
+    customOpts = list(
+      csv = list(name = "Download to CSV",
+                    callback = htmlwidgets::JS(
+                      "function (key, options) {
+                         var csv = csvString(instance);
+
+                         var link = document.createElement('a');
+                         link.setAttribute('href', 'data:text/plain;charset=utf-8,' +
+                           encodeURIComponent(csv));
+                         link.setAttribute('download', filename);
+
+                         document.body.appendChild(link);
+                         link.click();
+                         document.body.removeChild(link);
+                       }"))))
+
+## ----fig.height = 3------------------------------------------------------
+DF = data.frame(val = 1:10,
+                bool = TRUE,
+                big = LETTERS[1:10],
+                small = factor(letters[1:10]),
+                dt = seq(from = Sys.Date(), by = "days", length.out = 10),
+                stringsAsFactors = FALSE)
+
+rhandsontable(DF, search = TRUE) %>%
+  hot_context_menu(
+    customOpts = list(
+      search = list(name = "Search",
+                    callback = htmlwidgets::JS(
+                      "function (key, options) {
+                         var srch = prompt('Search criteria');
+
+                         this.search.query(srch);
+                         this.render();
+                       }"))))
 
 ## ----fig.height = 3------------------------------------------------------
 DF = data.frame(int = 1:10, float = rnorm(10), cur = rnorm(10) * 1E5,
